#include "picture_publisher.h"
#include "transfer.h"
#include "cl_settings.h"

_LIT(KClassName, "CPicturePublisherImpl");

class CPicturePublisherImpl : public CPicturePublisher, public MContextBase,
	public CCheckedActive, public i_status_notif, public MSettingListener  {
public:
	~CPicturePublisherImpl();
private:
	CPicturePublisherImpl(MApp_context& Context, i_status_notif& notif);
	void ConstructL(const TDesC& Path, const TDesC& Files, TInt Setting);

	void CheckedRunL();
	TInt CheckedRunError(TInt aError);
	void DoCancel();

	virtual void register_source(const TDesC&, const TDesC& initial_value, const TTime&);
	virtual void new_value(log_priority priority, const TDesC&, const TDesC& value, const TTime&);
	virtual void unregister_source(const TDesC&, const TTime&);
	virtual const TDesC& name() const;

	virtual void finished();
	virtual void error(const TDesC& descr);
	virtual void status_change(const TDesC& status);

	virtual void SettingChanged(TInt Setting);

	void Transfer();
	bool IsEnabled();
	void StartL();
	void Stop();

	i_status_notif& iNotif;
	CFtpTransfer*	iTransferer;
	TBuf<200>	iLoc;
	bool		iQueued;
	bool		iTransfering;
	bool		iNoPics;
	TFileName	iPath;
	TBuf<30>	iFiles;
	TInt		iSetting;

#ifdef __WINS__test
	RTimer		iTimer;
#endif

	friend class CPicturePublisher;
};

CPicturePublisher* CPicturePublisher::NewL(MApp_context& Context, i_status_notif& notif,
					const TDesC& Path, const TDesC& Files, TInt Setting)
{
	auto_ptr<CPicturePublisherImpl> ret(new (ELeave) 
		CPicturePublisherImpl(Context, notif));
	ret->ConstructL(Path, Files, Setting);
	return ret.release();
}

CPicturePublisherImpl::~CPicturePublisherImpl()
{
	Settings().CancelNotifyOnChange(iSetting, this);
	Stop();
#ifdef __WINS__test
	iTimer.Close();
#endif
}

CPicturePublisherImpl::CPicturePublisherImpl(MApp_context& Context, i_status_notif& notif) :
	MContextBase(Context), CCheckedActive(EPriorityNormal, KClassName), iNotif(notif)
{
}

void CPicturePublisherImpl::ConstructL(const TDesC& Path, const TDesC& Files, TInt Setting)
{
	iPath=Path; iFiles=Files; iSetting=Setting;
	Settings().NotifyOnChange(iSetting, this);
#ifdef __WINS__test
	iTimer.CreateLocal();
#endif
	CActiveScheduler::Add(this);

	if (IsEnabled()) {
		StartL();
	}
}

void CPicturePublisherImpl::Stop()
{
	Cancel();
	delete iTransferer;
}

void CPicturePublisherImpl::StartL()
{
	Stop();
	iTransferer=CFtpTransfer::NewL(AppContext(), this,
		iPath,
		_L(""), CFtpTransfer::HTTP);
	iTransferer->add_filesL(iFiles, false);
#ifndef __WINS__test
	Fs().NotifyChange(ENotifyEntry, iStatus, iPath);
#else
	iTimer.After( iStatus, 30*1000*1000 );
#endif
	SetActive();
}

bool CPicturePublisherImpl::IsEnabled()
{
	TBool enabled;
	if ( Settings().GetSettingL(iSetting, enabled) && enabled) return true;
	return false;
}

void CPicturePublisherImpl::SettingChanged(TInt)
{
	if (IsEnabled()) {
		TRAPD(err, StartL());
		if (err!=KErrNone) {
			TBuf<100> msg;
			msg.Format(_L("Error starting PicturePublisher: %d"), err);
			iNotif.error(msg);
		}
	} else {
		Stop();
	}
}
			

void CPicturePublisherImpl::CheckedRunL()
{
	Transfer();
#ifndef __WINS__test
	Fs().NotifyChange(ENotifyEntry, iStatus, iPath);
#else
	iTimer.After( iStatus, 10*1000*1000 );
#endif
	SetActive();
}

void CPicturePublisherImpl::Transfer()
{
	if (iTransfering) {
		iQueued=true;
	} else {
		iTransfering=true;
		iQueued=false;
		iNoPics=true;
		if (iLoc.Length()==0) iLoc=_L("Unknown");
		TTime now; now.HomeTime(); now-=TTimeIntervalMinutes(2);
		iTransferer->TransferFiles(iLoc, now);
		iNoPics=false;
	}
}

TInt CPicturePublisherImpl::CheckedRunError(TInt aError)
{
	return aError;
}

void CPicturePublisherImpl::DoCancel()
{
#ifndef __WINS__test
	Fs().NotifyChangeCancel();
#else
	iTimer.Cancel();
#endif
}

void CPicturePublisherImpl::register_source(const TDesC&, const TDesC& initial_value, const TTime&)
{
	if (initial_value.Length()>0) iLoc=initial_value;
}

void CPicturePublisherImpl::new_value(log_priority priority, const TDesC&, const TDesC& value, const TTime&)
{
	if (priority!=Mlogger::VALUE) iLoc=_L("");
	else iLoc=value;
}

void CPicturePublisherImpl::unregister_source(const TDesC&, const TTime&)
{
}

const TDesC& CPicturePublisherImpl::name() const
{
	return KClassName;
}

void CPicturePublisherImpl::finished()
{
	if (! iNoPics ) {
		iNotif.status_change(_L("uploaded pics"));
	}
	iTransfering=false;
	if (iQueued) Transfer();
}

void CPicturePublisherImpl::error(const TDesC& descr)
{
	iNotif.error(descr);
	iTransfering=false;
	if (iQueued) Transfer();
}

void CPicturePublisherImpl::status_change(const TDesC& status)
{
	iNotif.status_change(status);
}
